package nl.sbdeveloper.mctpaudio.radio;

import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.UnsupportedTagException;
import nl.sbdeveloper.mctpaudio.MCTPAudio;
import nl.sbdeveloper.mctpaudio.api.maps.SongList;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.utils.HeadUtil;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitTask;
import org.json.simple.JSONObject;

import java.io.IOException;

/* Copyright (C) McThemeParks - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Stijn Bannink <stijnbannink23@gmail.com>, July 2020
 */

public class Playlist {
    private SongList<String> playList = new SongList<>();
    private final SongList<String> playedList = new SongList<>();

    private boolean running = false;
    private BukkitTask currentTimer;

    /**
     * Create a new PlayList object
     * Starts the playlist
     */
    public Playlist() {
        start();
    }

    /**
     * Start this playlist
     */
    public void start() {
        for (String URL : MCTPAudio.getPlugin().getConfig().getStringList("RadioSongs")) {
            addSong(URL);
        }

        Bukkit.getScheduler().runTaskAsynchronously(MCTPAudio.getPlugin(), this::nextSong);

        running = true;
    }

    /**
     * Stop the playlist
     */
    public void stop() {
        playList.clear();

        if (currentTimer != null) {
            currentTimer.cancel();
            currentTimer = null;
        }

        running = false;
    }

    /**
     * Add a song by the url
     *
     * @param url The song url (mp3)
     */
    public void addSong(String url) {
        playList.add(url);
        playList.shuffle();
    }

    /**
     * Go to the next song
     */
    public void nextSong() {
        if (currentTimer != null) return;

        if (playList.isEmpty()) {
            //All numbers played
            playList = (SongList<String>) playedList.clone(); //Copy a clone.
            playedList.clear();
        }

        //Get song
        String nextURL = playList.getRandom();
        if (nextURL == null) return;

        playList.remove(nextURL);
        playedList.add(nextURL);

        JSONObject data;
        for (Player p : Bukkit.getOnlinePlayers()) {
            data = new JSONObject();

            if (!PinManager.hasPin(p.getUniqueId())) continue;

            data.put("task", "RADIO");
            data.put("path", nextURL);
            data.put("uuid", p.getUniqueId().toString().replace("-", ""));
            MCTPAudio.getClient().sendData(data);
        }

        int ticks;
        try {
            ticks = HeadUtil.getTicksOfFile(nextURL);
        } catch (IOException | InvalidDataException | UnsupportedTagException e) {
            e.printStackTrace();
            nextSong();
            return;
        }

        if (ticks == 0) {
            Bukkit.getLogger().info("0 ticks");
            nextSong();
            return;
        }

        Bukkit.getLogger().info("Started song with duration: " + ticks / 20 + " sec.");

        currentTimer = Bukkit.getScheduler().runTaskLaterAsynchronously(MCTPAudio.getPlugin(), () -> {
            currentTimer = null;
            nextSong();
        }, ticks);

        //Shuffle playlist again
        playList.shuffle();
    }

    /**
     * Check if the playlist is running
     *
     * @return true if running
     */
    public boolean isRunning() {
        return running;
    }
}
